rm(list=ls())
source("fdrCalc.R")

####################################################
#
# load in the psychology replication project data:
#
####################################################

rpp.o <- read.csv("rpp_data.csv")
rpp <- subset(rpp.o, subset=(T_df1..O.==1))

# rectify correlations outside the boundaries
rpp <- within(rpp,effcor <- pmin(pmax(0,T_r..O.),1) )

# create "replicable" variable
rpp <- within(rpp, replicable <- as.numeric(Findings.similarity..R.) >=6)

#Fisher Tranformation for effect size and sd
rpp$N <- as.numeric(rpp$T_df2..O.)
rpp <- subset(rpp, subset= (is.na(N)==F) )
rpp<-within(rpp,sd<-1/sqrt(N-3))
rpp<-within(rpp,beta.hat<-0.5*log((1+effcor)/(1-effcor)))

# final RPP data set for analysis
dat <- subset(rpp, select=c(beta.hat, sd, replicable, N))
dat$replicable <- as.numeric(dat$replicable)

# calculate p-values using degrees of freedom
dat$p <- 2*(1 - pt( dat$beta.hat/dat$sd, df=dat$N))

# subset only conventionally significant results (0.05, one-tailed / 0.1, two-tailed)
dat.pub <- subset(dat, subset=(p <= 0.1))
dim(dat.pub)



##############################################
#
# Analyze psych replication project results
#
#############################################



# vectorize the fdrCalc function along the result elements
fdr.calc.v <- Vectorize(fdr.calc, c("beta.hat", "beta.sd", "beta.df"))

# function to determine best-fitting alt.prob given a fixed bias
fdr.optim.obj <- function(input, dat, bias=0){
  
  input <- as.matrix(input)
  if( input >= 1 | input <= 0 ){return(1e10)}
  
  # calculate the FDR for all inputted bias and alt.prob values
  temp <- fdr.calc.v(dat$beta.hat, dat$sd, beta.df=dat$N, p.test=0.1, bias=bias,
                     alt.prob=input, unif.cen=0, unif.bound = 3, print.msg=F)
  
  # compute squared differences of observed FDR (temp) from target FDR (the "replicable" vector in dat)
  out <- sum( (mean(temp) - (1-mean(dat$replicable)) )^2 )
  
  #print(out)
  
  # return results 
  return(out)
  
}

psych.calc <- function(bias, dat){

  alt.prob <- optim(par=0.5, fn=fdr.optim.obj, method="Brent", lower=0.001, upper=0.999, 
                          control=list(trace=0), dat = dat.pub, bias = bias)$par
  return(1-alt.prob)
  
}

bias.vec <- seq(from=0, to=1, by=0.01)

library(parallel)
cl <- makeCluster(4)
clusterExport(cl=cl, varlist=c("fdr.calc", "fdr.calc.v", "fdr.optim.obj", "dat.pub"))
clusterSetRNGStream(cl, iseed=123456)

results <- clusterApply(cl=cl, x=bias.vec, fun=psych.calc, dat = dat.pub)

stopCluster(cl)

save.image(file="RPP-analysis-image.RData")

pdf(file="RPP-bias-null-estimates.pdf", width=5, height=5.5)

pr.null.vec <- unlist(results)
plot(pr.null.vec ~ bias.vec, type="l", xlab = "researcher bias in beta estimates", 
     ylab = "estimated Pr(null is true)", 
     main = c("prior Pr(null) / bias values consistent with", "FDR in OSC Psychology Replication Project"))

dev.off()


pdf(file="RPP-bias-null-estimates-poster.pdf", width=5, height=5.5)

pr.null.vec <- unlist(results)
plot(pr.null.vec ~ bias.vec, type="l", xlab = "researcher bias in beta estimates", 
     ylab = "estimated prior Pr(null is true)", 
     main = c("Figure 1: prior Pr(null) / bias values consistent with", "FDR in OSC Psychology Replication Project"))

dev.off()


