#' Calculate False Discovery Rate for a Published Result
#'
#' This software takes a published result (coefficient, standard error, and degrees of freedom) as input and returns the corresponding false discovery rate (FDR) as a function of the statistical test used and the prior expectation of null results. By calculating this FDR for every value of the prior expectation between 0 and 1, the software also (optionally) creates a curve relating FDR to prior probability.
#' 
#'
#' @param beta.hat An estimated quantity (e.g., coefficient or marginal effect).
#' @param beta.sd An estimated uncertainty for beta.hat.
#' @param beta.df The degrees of freedom for the estimate of beta.hat.
#' @param p.test The probability of a statistically significant result under the null (i.e., the size/alpha value of a statistical significance test).
#' @param bias The assumed degree of researcher-induced bias in size and power of a statistical significance test, as an additive component of beta.hat.
#' @param alt.prob The probability that the null hypothesis is false (between 0 and 1, default = 0.5).
#' @param print.msg Should the FDR and critical prior values be printed? Must be = TRUE or FALSE.
#' @param dist The distribution of the prior density; must = "uniform" or "normal."
#' @param unif.cen The center of the uniform prior density (same scale as beta.hat).
#' @param unif.bound Distance between the center of uniform density and the upper/lower boundary, as a multiple of beta.hat.
#' @param norm.cen The center of the normal prior density (same scale as beta.hat).
#' @param norm.bound Standard deviation of the normal prior density, as a multiple of beta.hat.
#'
#' @return 
#' \item{fdr}{The calculated false discovery rate.}
#' @author Justin Esarey
#' @examples
#' \dontrun{
#' 
#' 
#' ## example: aggregate personal savings rates
#' ## ?LifeCycleSavings
#' 
#' # run model
#' fm1 <- lm(sr ~ pop15 + pop75 + dpi + ddpi, data = LifeCycleSavings)
#' summary(fm1)
#' 
#' # save the results of the model
#' coefs <- coefficients(summary(fm1))
#' 
#' # calculate FDR for effect of growth in per capita disposable income
#' # on savings/disposable income ratio, Pr(null is false) = 50%
#' fdr.calc(coefs[5,1], coefs[5,2], fm1$df)
#' 
#' # Pr(null is false) = 70%
#' fdr.calc(coefs[5,1], coefs[5,2], fm1$df, alt.prob = 0.3)
#' 
#' #' # Pr(null is false) = 90%
#' fdr.calc(coefs[5,1], coefs[5,2], fm1$df, alt.prob = 0.1)
#' 
#' 
#' }
#' @rdname fdr.calc
#' @references Benjamini, Y., & Hochberg, Y. (1995). Controlling the False Discovery Rate: A Practical and Powerful Approach to Multiple Testing. Journal of the Royal Statistical Society. Series B (Methodological), 57(1), 289-300. <URL:http://www.jstor.org/stable/2346101>.
#' @import stats
#' @import graphics
#' @import datasets
#' @export
#' 


fdr.calc <- function(beta.hat, beta.sd, beta.df, p.test="result", bias=0,
                     print.msg = TRUE, dist="uniform", alt.prob = 0.5,
                     unif.cen = 0, unif.bound = 3,
                     norm.cen = 0, norm.bound = 1){

  # p-value of the original result, two-tailed
  # gives Pr(significant | null)
  if(p.test == "result"){
    p.type <- "result"
    p.test <- max( (2*(1-pt(abs(beta.hat/beta.sd), df=beta.df))), 0.001)
  }else{
    p.type <- "input"
  }
  
  # save p.test for print on graph
  p.test.display <- p.test

  # account for researcher-induced bias term
  if(p.type == "result"){
    p.test <- max( (2*(1 - pt( (( abs(beta.hat) - bias)/beta.sd) , df=beta.df) ) ), 1e-8)
  }else{
    p.test <- min( max( (2*(1 - pt( ( abs(qt(p.test/2, df=beta.df)) - bias/beta.sd ), df=beta.df) ) ), 1e-8), 1)
  }
    
  if(dist=="uniform"){
    
    # check for valid domain
    if(is.numeric(unif.cen) == FALSE | is.numeric(unif.bound) == FALSE ){
      stop("unif.cen and unif.bound must be numeric")
    }
    
    # probability of a statistically significant result under the alternative hypothesis
    # for a uniform density
    pr_density<-function(b.hat, beta.sd, beta.df){
      
      unif.low = unif.cen - unif.bound * abs(beta.hat)
      unif.hi = unif.cen + unif.bound * abs(beta.hat)

      # Pr(significant | b.hat, alt)
      pr.upper <- ( 1 - pt( ( qt( (1 - (p.test/2) ), df = beta.df) - b.hat/beta.sd ), df=beta.df )  )  
      pr.lower <- ( pt( ( (-1)*qt( (1 - (p.test/2) ), df = beta.df) - b.hat/beta.sd ), df=beta.df )  ) 
      pr.sig.alt <- pr.upper+pr.lower
      
      # Pr(significant | b.hat, alt) * Pr(b.hat | alt)
      return( pr.sig.alt*dunif(b.hat, min=unif.low, max=unif.hi)  )

    }
    
  }
  
  if(dist=="normal"){
    
    # check for valid domain
    if(is.numeric(norm.cen) == FALSE | is.numeric(norm.bound) == FALSE ){
      stop("unif.cen and unif.bound must be numeric")
    }
    
    # probability of a statistically significant result under the alternative hypothesis
    # for a normal density
    pr_density<-function(b.hat, beta.sd, beta.df){
      
      # Pr(significant | b.hat, alt)
      pr.upper <- ( 1 - pt( ( qt( (1 - (p.test/2) ), df = beta.df) - b.hat/beta.sd ), df=beta.df )  )  
      pr.lower <- ( pt( ( (-1)*qt( (1 - (p.test/2) ), df = beta.df) - b.hat/beta.sd ), df=beta.df )  ) 
      pr.sig.alt <- pr.upper+pr.lower
      
      norm.mean = norm.cen
      norm.sd = norm.bound * abs(beta.hat)
      
      # Pr(significant | b.hat, alt) * Pr(b.hat | alt)
      return( pr.sig.alt*dnorm(b.hat, mean=norm.mean, sd=norm.sd) )
      
    }
    
  }
  
  
  if(dist != "uniform" & dist != "normal" ){
    stop("dist must be 'uniform' or 'normal'")
  }
  
  # integrate out Pr(b.hat) to get Pr(significant | alt)
  q<-integrate(pr_density,lower=(-Inf), upper=Inf, beta.sd=beta.sd, beta.df=beta.df)$value
  
  # ratio of statistically significant null results to statistically significant non-null results
  # = ( Pr(significant | null) * Pr(null) ) / [ ( Pr(significant | alt) * Pr(alt) ) + ( Pr(significant | null) * Pr(null) )]
  pwr.ratio <- (p.test*(1-alt.prob)) / ( (q*(alt.prob)) + (p.test*(1-alt.prob)))
  
  if(print.msg==TRUE){

    cat("\n", "Calculated False Discovery Rate: ", signif(pwr.ratio, 3), "\n", "\n", sep="")
  
  }
  
  return(invisible(pwr.ratio))
}